@@ -481,6 +481,8 @@ otherprog:
 			exit(EX_UNAVAILABLE);
 		}
 		if (vfd >= 0) {
+			if (fork() > 0) exit(EX_OK); /* Let the child to do
+							the trace printout */
 			vfp = fdopen(vfd, "r");
 			while (vfp != NULL) {
 				while (fgets(buf, sizeof buf, vfp) != NULL)
