@@ -4,7 +4,7 @@
  */
 
 #include "mailer.h"
-#include <signal.h>
+#include <signal.h> /* used for kill(2) */
 
 #ifndef	_IOFBF
 #define	_IOFBF	0
