@@ -591,6 +591,7 @@ db(dbname, key)
 	struct search_info si;
 	struct cache tce;
 	u_char kbuf[BUFSIZ];	/* XX: */
+	u_char *kbend = kbuf + sizeof(kbuf) -1;
 
 	if (key == NULL || *key == '\0') {
 		fprintf(stderr,
@@ -610,19 +611,21 @@ db(dbname, key)
 	realkey = NULL;
 	if (dbip->flags & DB_MAPTOLOWER) {
 		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
-			if (isascii(*realkey) && isupper(*realkey))
-				*cp++ = tolower(*realkey);
-			else
-				*cp++ = *realkey;
+		  if (cp < kbend)
+		    if (isascii(*realkey) && isupper(*realkey))
+		      *cp++ = tolower(*realkey);
+		    else
+		      *cp++ = *realkey;
 		}
 		*cp = '\0';
 		key = kbuf;
 	} else if (dbip->flags & DB_MAPTOUPPER) {
 		for (cp = kbuf, realkey = key; *realkey; ++realkey) {
-			if (isascii(*realkey) && islower(*realkey))
-				*cp++ = toupper(*realkey);
-			else
-				*cp++ = *realkey;
+		  if (cp < kbend)
+		    if (isascii(*realkey) && islower(*realkey))
+		      *cp++ = toupper(*realkey);
+		    else
+		      *cp++ = *realkey;
 		}
 		*cp = '\0';
 		key = kbuf;
