@@ -390,7 +390,7 @@ vtxredo(spl)
 		vp->ce.argv = NULL;
 		vp->ce.retries = NULL;
 		vp->ce.nretries = 0;
-		vtxdo(vp, rrcf_head);
+		vtxdo(vp, rrcf_head, NULL);
 	}
         return 0;
 }
