@@ -106,7 +106,6 @@ extern void deletemsg __((char *, struct ctlfile *));
 extern void detach __((void));
 extern void die __((int rc, char *reason));
 extern void doagenda __((void));
-extern void ipcpause __((int pollflag));
 extern void killprevious __((int signal, char *pidfile));
 extern void link_in __((int flag, struct vertex *vp, u_char *s));
 extern void mklink __((char *from, char *to));
@@ -281,8 +280,11 @@ main(argc, argv)
 		now = time(NULL);
 		printf("%s: scheduler daemon (%s)\n\tstarted at %s\n",
 		       progname, Version, (char *)rfc822date(&now));
-		killprevious(SIGTERM, pidfile);
 	}
+	/* Actually we want this to act as daemon,
+	   even when not in daemon mode.. */
+	killprevious(SIGTERM, pidfile);
+
 	for (i = 0; i < SIZE_L; ++i)
 		spt_mesh[i] = sp_init();
 #ifdef  LOG_PID
@@ -368,8 +370,7 @@ main(argc, argv)
 				tp = rereadconfig(tp, config);
 				rereadcf = 0;
 			}
-			/* if (numkids > 0) */
-			mux(0);
+			mux(0); /* Slow check */
 		}
 		canexit = 1; /* may be possible to move this after doagenda() */
 		/* wait for an alarm call */
@@ -382,8 +383,7 @@ main(argc, argv)
 				tp = rereadconfig(tp, config);
 				rereadcf = 0;
 			}
-			/* ipcpause(0); */
-			mux(0);
+			mux(0); /* Slow check */
 		}
 	}
 #ifdef	MALLOC_TRACE
@@ -466,8 +466,7 @@ syncweb(dir, nukefile, cehead)
 	  if (mustexit)
 	    break;
 
-	  /* ipcpause(1); */
-	  mux(1);
+	  mux(1); /* Fast check */
 
 	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0])) {
 	    if (strcmp(dir, ".") == 0)
