@@ -63,6 +63,7 @@ struct ctlfile {
 	time_t	ctime;		/* when this file was created (more or less) */
 	int	haderror;	/* some errors/diagnostics need processing */
 	struct vertex	*head;		/* head of the list of groups */
+	struct vertex	*atspool;	/* the vertex currently scheduled */
 	int	nlines;		/* number of lines/entries in the file */
 	char	*contents;	/* the control file as copied into memory */
 	char	*logident;	/* identification for log entries */
@@ -100,6 +101,7 @@ struct procinfo {
 struct vertex {		
 	struct ctlfile	*cfp;		/* control file containing this group */
 	int		linkcnt;
+	char		*linkpath;	/* Scheduled file */
 	struct web	*orig[SIZE_L];	/* original names (channel,host,etc) */
 	struct vertex	*next[SIZE_L];	/* next group with same L_? */
 	struct vertex	*prev[SIZE_L];	/* previous group with same L_? */
