@@ -15,7 +15,7 @@
 #include NDIR_H
 #include "zmsignal.h"
 
-int scheduler_nofiles = 32; /* Will be filled below */
+int scheduler_nofiles = -1; /* Will be filled below */
 
 #ifdef	USE_UNIONWAIT
 #include <sys/wait.h>
@@ -112,6 +112,9 @@ tscan(spl)
 	if (verbose)
 	  printf("  tscan() path=\"%s\"\n",path);
 #endif
+	if (verbose)
+	  printf("  tscan() path=\"%s\"\n",cfp->atspool->linkpath);
+
 	t_files[t_n++] = cfp->atspool->linkpath;
 	cfp->mark    = V_NONE;
 	cfp->atspool = NULL;
@@ -290,8 +293,9 @@ runcommand(filearr, nelem, argv, vhead, chwp, howp)
 #if	1
 	  /* ... if detach() did its job, there shouldn't be any! */
 	  /* ... no, the 'querysock' is there somewhere!   */
-	  topfileno = resources_query_nofiles();
-	  for (i = 3; i < topfileno; ++i)
+	  if (scheduler_nofiles < 1)
+	    scheduler_nofiles = resources_query_nofiles();
+	  for (i = 3; i < scheduler_nofiles; ++i)
 	    close(i);
 #else
 	  close(to[1]);
@@ -353,7 +357,9 @@ stashprocess(pid, fd, chwp, howp, vhead)
 		  pid, fd, chwp ? chwp->name : "nil",
 		  howp ? howp->name : "nil");
 	if (cpids == NULL) {
-	  scheduler_nofiles = i = resources_query_nofiles();
+	  if (scheduler_nofiles < 1)
+	    scheduler_nofiles = resources_query_nofiles();
+	  i = scheduler_nofiles;
 	  cpids = (struct procinfo *)
 	    emalloc((unsigned)(i * sizeof (struct procinfo)));
 	  while (--i >= 0)
@@ -544,7 +550,7 @@ int quickcheck;
 	maxf = -1;
 	FD_ZERO(&mask);
 	if (cpids != NULL)
-	  for (i = 0; i < FD_SETSIZE ; ++i)
+	  for (i = 0; i < scheduler_nofiles ; ++i)
 	    if (cpids[i].pid != 0) {
 	      FD_SET(i, &mask);
 	      maxf = i;
@@ -602,6 +608,7 @@ int quickcheck;
 	/* fprintf(stderr, "return from mux\n"); */
 }
 
+#if 0 /* ipcpause() NOT in use, all done now inside  mux() */
 void
 ipcpause(pollflag)
 int pollflag;
@@ -653,6 +660,7 @@ int pollflag;
 	  pause();
 #endif	/* AF_INET */
 }
+#endif /* ipcpause() NOT in use! */
 
 void
 queryipcinit()
@@ -712,7 +720,7 @@ int quickcheck; /* Is always a quickcheck..  Aurgh the induced load.. */
 	 * simulation USG supplies, but it ain't worth the hassle.
 	 */
 	if (cpids != NULL)
-	  for (fd = 0; fd < FD_SETSIZE ; ++fd)
+	  for (fd = 0; fd < scheduler_nofiles ; ++fd)
 	    if (cpids[fd].pid != 0)
 	      readfrom(fd);
 }
@@ -787,6 +795,7 @@ readfrom(fd)
 	      bufsize += 1024;
 	      pcp = buf = erealloc(buf,bufsize);
 	      cp = buf + (bufsize - 1024);
+	      *cp = '\0';
 	    } else {
 	      memcpy(buf, pcp, cp-pcp);
 	      cp = buf + (cp-pcp);
@@ -819,7 +828,7 @@ readfrom(fd)
 	  if (cpids[fd].pid != 0) {
 	    cpids[fd].carryover = emalloc(cp-pcp+1);
 	    memcpy(cpids[fd].carryover, pcp, cp-pcp);
-	    *(cpids[fd].carryover+(cp-pcp)) = '\0';
+	    cpids[fd].carryover[cp-pcp] = '\0';
 	  } else
 	    fprintf(stderr,
 		    "HELP! Lost %d bytes (n=%d/%d, off=%d): '%s'\n",
@@ -884,7 +893,7 @@ system(name)
 		    } else if (r == pid) {
 		      break;
 		    }
-		    for(i = 0; i < FD_SETSIZE; i++) {
+		    for(i = 0; i < scheduler_nofiles; i++) {
 		      if (cpids[i].pid == r) {
 			cpids[i].pid = -r;
 			break;
