@@ -95,7 +95,7 @@ update(diagnostic)
 	  message = cp;
 	}
 	if (verbose)
-	  printf("diagnostic: %d/%d/%s/%s\n",inum,offset,notary,type);
+	  printf("diagnostic: %d/%d\t%s\t%s\n",inum,offset,notary,type);
 
 	for (dcp = &diags[0]; dcp->name != NULL; ++dcp) {
 	  if (cistrcmp(dcp->name, type) == 0) {
