@@ -11,6 +11,11 @@ char *VerbID = "ZMailer SMTP server %s";
 char *Copyright = "Copyright 1990 Rayan S. Zachariassen";
 char *Copyright2= "Copyright 1991-1994 Matti Aarnio";
 
+/* Timing parameters -- when expired, session is killed ... */
+
+#define SMTP_COMMAND_ALARM_IVAL 1200 /* 20 minutes.. */
+#define SMTP_DATA_TIME_PER_LINE  600 /* 10 minutes of life.. */
+
 /*
  * The smtpserver connects to the router to ask it various questions, like,
  * is this a valid address?  What is the alias expansion of that? etc.
@@ -84,8 +89,8 @@ char *Copyright2= "Copyright 1991-1994 Matti Aarnio";
  * when we are being run by the 4.2 inetd is included by defining
  * CHECK42INETD.  Should be deleted at the earliest possible opportunity.
  */
-#define	CHECK42INETD	/* heuristics to detect the 4.2 inetd */
-
+/* #define	CHECK42INETD	/* heuristics to detect the 4.2 inetd */
+	/* [mea@utu.fi] - 4-Feb-95, I disabled this.. */
 
 typedef enum {	Null, Hello, Mail, MailOrHello, Recipient,
 		RecipientOrData, Data, Send, SendOrMail,
@@ -207,10 +212,6 @@ unsigned short	auth_flag = 0;
 #  define	AUTH_TIMEOUT	5
 # endif	/* AUTH_TIMEOUT */
 char		auth_username[MAXHOSTNAMELEN+1];
-unsigned long	auth_inlocal;
-unsigned long	auth_inremote;
-unsigned short	auth_local;
-unsigned short	auth_remote;
 extern void	setrfc1413auth();
 #endif	/* USE_AUTH */
 
@@ -246,6 +247,7 @@ extern int optind;
 extern char *optarg;
 
 extern char *strchr();
+extern char *strrchr();
 
 #ifdef	lint
 #undef	putc
@@ -282,7 +284,7 @@ int insecure;
 int
 main(argc, argv)
 	int argc;
-	char *argv[];
+	char **argv;
 {
 	int inetd, errflg, c, raddrlen, s, msgfd, version, i;
 	char *mailshare, path[1024];
@@ -305,6 +307,7 @@ main(argc, argv)
 				       progname, myhostname, strerror(errno));
 		exit(1);
 	}
+	/* optarg = NULL; */
 	while ((c = getopt(argc, argv, "aBigl:np:L:M:P:R:s:Vv")) != EOF) {
 		switch (c) {
 #ifdef	USE_AUTH
@@ -330,15 +333,13 @@ main(argc, argv)
 			skeptical = 0;
 			break;
 		case 'l':	/* log file */
-			logfile = emalloc(strlen(optarg)+1);
-			strcpy(logfile, optarg);
+			logfile = strdup(optarg);
 			break;
 		case 'n':	/* running under inetd */
 			inetd = 1;
 			break;
 		case 's':	/* checking style */
-			style = emalloc(strlen(optarg)+1);
-			strcpy(style, optarg);
+			style = strdup(optarg);
 			break;
 		case 'L':	/* Max LoadAverage */
 			maxloadavg = atoi(optarg);
@@ -351,18 +352,17 @@ main(argc, argv)
 			break;
 #endif	/* USE_INET */
 		case 'R':	/* router binary used for verification */
-			routerprog = emalloc(strlen(optarg)+1);
-			strcpy(routerprog, optarg);
+			routerprog = strdup(optarg);
 			break;
 		case 'P':
-			postoffice = emalloc(strlen(optarg)+1);
-			strcpy(postoffice, optarg);
+			postoffice = strdup(optarg);
 			break;
 		case 'V':
 			prversion("smtpserver");
 			exit(0);
 			break;	/* paranoia */
 		default:
+fprintf(stderr,"%s: Unknown option, c=%d ('%c')\n",progname,c,c);
 			++errflg;
 			break;
 		}
@@ -475,19 +475,16 @@ main(argc, argv)
 			exit(1);
 		}
 		if (port <= 0) {
+			struct servent *service;
 #ifdef	IPPORT_SMTP
 			port = htons(IPPORT_SMTP);
-#else	/* !IPPORT_SMTP */
-			struct servent *service;
-
+#endif	/* !IPPORT_SMTP */
 			if ((service = getservbyname("smtp", "tcp")) == NULL) {
 				fprintf(stderr,
-					"%s: no SMTP service entry\n",
+					"%s: no SMTP service entry, using default\n",
 					progname);
-				exit(1);
-			}
-			port = service->s_port;
-#endif	/* IPPORT_SMTP */
+			} else
+			  port = service->s_port;
 		}
 		sad.sin_family = AF_INET;
 		sad.sin_addr.s_addr = INADDR_ANY;
@@ -677,13 +674,19 @@ timedout()
 {
 	reporterr(0,"SMTP protocol timed out");
 
+	/* If there is something going on, kill the file.. */
+	if (mfp != NULL)
+		mail_abort(mfp);
+	mfp = NULL;
+
 	if (routerpid > 0)
 		killr(routerpid);
 	exit(0);
 }
 
 SIGNAL_TYPE
-reaper()
+reaper(sig)
+int sig;
 {
 #ifdef	USE_UNIONWAIT
 	union wait status;
@@ -1253,11 +1256,10 @@ smtpserver(insecure)
 		fflush(logfp);
 	}
 
-#define SMTP_ALARM_IVAL (20*60) /* 20 minutes.. */
 	while (1) {
 
 		/* Alarm processing on the SMTP protocol channel */
-		alarm(SMTP_ALARM_IVAL);
+		alarm(SMTP_COMMAND_ALARM_IVAL);
 
 		*buf = 0;
 		s = buf;
@@ -1387,10 +1389,10 @@ smtpserver(insecure)
 				break;
 			}
 			type(354, (char *)NULL);
-			alarm(60*60*24);	/* one day should be enough! */
+			/* We set alarm()s inside the mvdata() */
 			bad8bithdr = 0;
 			filsiz = mvdata(mfp,&bad8bithdr);
-			alarm(0);	/* cancel alarm */
+			alarm(0);	/* cancel the alarm() */
 			tell = ftell(mfp);
 			if (fstat(fileno(mfp), &stbuf) < 0)
 				inum = 0;
@@ -1399,25 +1401,31 @@ smtpserver(insecure)
 
 			if (bad8bithdr) {
 				mail_abort(mfp);
+				mfp = NULL;
 				type(452,"Message header had 8-bit character(s) outside Subject!");
 			} else if (feof(stdin) || ferror(stdin)) {
 				/* [mea@utu.fi] says this can happen */
 				mail_abort(mfp);
+				mfp = NULL;
 				reporterr(tell, "premature EOF on DATA input");
 				return;
 			} else if (ferror(mfp)) {
 				type(452, (char *)NULL);
 				clearerr(mfp);
 				mail_abort(mfp);
+				mfp = NULL;
 				reporterr(tell, "message file error");
 			} else if (maxsize > 0 && filsiz > maxsize) {
 				mail_abort(mfp);
+				mfp = NULL;
 				type(552, "Message size exceeded fixed maximum size for acceptable email");
 
 			} else if (mail_close(mfp) == EOF) {
 				type(452, (char *)NULL);
+				mfp = NULL;
 				reporterr(tell, "message file close failed");
 			} else {
+				mfp = NULL;
 				type(250, "Roger");
 #ifdef	LOG_INFO
 				syslog(LOG_INFO,
@@ -1431,8 +1439,6 @@ smtpserver(insecure)
 					fflush(logfp);
 				}
 			}
-			fclose(mfp);	/* just in case */
-			mfp = NULL;
 			state = MailOrHello;
 			break;
 		case Reset:
@@ -1750,7 +1756,7 @@ mgets(buf, flagp, fp)
 		return NULL;
 	}
 
-	bufsize = 10;
+	bufsize = 16;
 	cp = buf = emalloc(bufsize);
 	while ((*cp = getc(fp)) != EOF) {
 		if (*cp == '\n') {
@@ -1766,7 +1772,7 @@ mgets(buf, flagp, fp)
 		if (bufsize - 2 < cp - buf) {
 			bufsize *= 2;
 			s = erealloc(buf, bufsize);
-			cp += s - buf;
+			cp  = s + (cp - buf);
 			buf = s;
 		}
 	}
@@ -2140,6 +2146,7 @@ mvdata(out,bad8bitp)
 {
 	register FILE *input;
 	register int c, state, *idxnum, *sts, endstate, cnt;
+	int timercnt = 1024;
 
 #ifdef NO_INCOMING_HEADER_PROCESSING
 	idxnum = indexnum+1;
@@ -2165,6 +2172,8 @@ mvdata(out,bad8bitp)
 	cnt = 0;
 	col = 0;
 
+	alarm(SMTP_DATA_TIME_PER_LINE);
+
 	/* ================ Input the email headers ================ */
 	/*	     and analyze them a bit (Precedence:)	     */
 	/*	  ... that only "Subject:" has 8-bit chars ...	     */
@@ -2175,6 +2184,12 @@ mvdata(out,bad8bitp)
 #if EOF != -1
 		if (c == EOF) return EOF;
 #endif
+		if (--timercnt == 0) {
+		  /* Re-arm for every kilobyte */
+		  alarm(SMTP_DATA_TIME_PER_LINE);
+		  timercnt = 1024;
+		}
+
 		if (0x80 & c) has8bit = 1;
 		if ((state = sts[state+idxnum[c]]) & ~0xff) {
 			if (state & O_) {
@@ -2243,6 +2258,11 @@ mvdata(out,bad8bitp)
 		if (c == EOF)		/* a little slower... */
 			return EOF;
 #endif
+		if (--timercnt == 0) {
+			/* Re-arm for every kilobyte */
+			alarm(SMTP_DATA_TIME_PER_LINE);
+			timercnt = 1024;
+		}
 		++cnt;
 		if ((state = sts[state+idxnum[c]]) & ~0xff) {
 			if (state & O_) {
@@ -2252,11 +2272,11 @@ mvdata(out,bad8bitp)
 				continue;
 			}
 			if (!ferror(out))
-			  putc((state>>8), out);
+				putc((state>>8), out);
 			state = (char)state;
 		}
 		if (!ferror(out))
-		  putc(c, out);
+			putc(c, out);
 	}
 	*bad8bitp = has8bitsum;
 	return cnt;
@@ -2268,12 +2288,17 @@ setrfc1413auth(sd)
 unsigned int sd;
 {
     char *cp;
+    struct in_addr	auth_inlocal;
+    struct in_addr	auth_inremote;
+    unsigned short	auth_local;
+    unsigned short	auth_remote;
+
     strncpy(auth_username, "UNKNOWN", MAXHOSTNAMELEN);
     if (auth_fd2(sd, &auth_inlocal, &auth_inremote,
 		 &auth_local, &auth_remote) == -1) {
 	    return;
     } else {
-	    cp = auth_tcpuser3(auth_inlocal, auth_inremote,
+	    cp = auth_tcpuser3(&auth_inlocal, &auth_inremote,
 			       auth_local,   auth_remote,
 			       AUTH_TIMEOUT);
 	    if ( cp != (char *) NULL ) {
