@@ -216,9 +216,11 @@ ctlopen(file, channel, host, exitflagp, selectaddr, matchrouter)
 	  getcwd(cwd,MAXPATHLEN);
 #endif
 	  sprintf(buf,
-		  "Cannot open control file \"%%s\" from \"%s\" for \"%%s/%%s\"as uid %d! (%%m)",
+		  "Cannot open control file \"%%s\" from \"%s\" for \"%%s/%%s\" as uid %d! (%%m)",
 		  cwd, geteuid());
 	  errno = e;
+	  if (host == NULL)
+	    host = "-";
 	  warning(buf, file, channel, host);
 	  return NULL;
 	}
