@@ -15,17 +15,16 @@
 
 #include <stdio.h>
 #include "hostenv.h"
-#include <sys/param.h>
-#include <sys/file.h>
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
 #include <sysexits.h>
 #include <varargs.h>
+#include <sys/param.h>
 #include <syslog.h>
 #include <fcntl.h>
+#include <sys/file.h>
 #include <sys/stat.h>
-#include NDIR_H
 #include "mail.h"
 #include "zmsignal.h"
 #include "ta.h"
@@ -281,6 +280,8 @@ main(argc, argv)
 	char *s;
 	int c, errflg, fd;
 	struct ctldesc *dp;
+	int matchhost = 0;	/* Match also the host-part when picking job */
+	char *host = NULL;	/* .. and what is my host ? */
 #if	defined(BIFF) || defined(RBIFF)
 	struct biffer *nbp;
 #endif
@@ -311,7 +312,7 @@ main(argc, argv)
 	errflg = 0;
 	logfile = NULL;
 	channel = CHANNEL;
-	while ((c = getopt(argc, argv, "abc:gl:rMV8")) != EOF) {
+	while ((c = getopt(argc, argv, "abc:gh:l:rMV8")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = optarg;
@@ -335,6 +336,10 @@ main(argc, argv)
 	  case 'g':		/* toggle file creation */
 	    creatflag = !creatflag;
 	    break;
+	  case 'h':
+	    host = strsave(optarg);
+	    matchhost = 1;
+	    break;
 	  case 'l':		/* log file */
 	    logfile = strsave(optarg);
 	    break;
@@ -347,7 +352,7 @@ main(argc, argv)
 	  }
 	}
 	if (errflg || optind != argc) {
-	  fprintf(stderr, "Usage: %s [-V8abr] [-c channel]\n",
+	  fprintf(stderr, "Usage: %s [-V8abr] [-c channel] [-h host]\n",
 		  argv[0]);
 	  exit(EX_USAGE);
 	}
@@ -374,7 +379,7 @@ main(argc, argv)
 	while (!getout && fgets(file, sizeof file, stdin)) {
 	  if (emptyline(file, sizeof file))
 	    break;
-	  dp = ctlopen(file, channel, (char *)NULL, &getout, NULL, NULL);
+	  dp = ctlopen(file, channel, host, &getout, NULL, NULL);
 	  if (dp == NULL)
 	    continue;
 	  if (verboselog) {
