@@ -469,10 +469,16 @@ main(argc, argv)
 	}
 	if (smtpstatus == EX_OK) {
 	  smtpstatus = smtpwrite(smtpfp, verboselog, 0, "QUIT");
+	}
+	/* Close the channel -- if it is open anymore .. */
+	if (smtp_isopen) {
 	  fclose(smtpfp[0]);
 	  fclose(smtpfp[1]);
-	  smtp_isopen = 0;
+	  if (logfp)
+	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
 	}
+	smtp_isopen = 0;
+
 	if (verboselog != NULL)
 		fclose(verboselog);
 	if (logfp != NULL)
@@ -521,11 +527,15 @@ process(dp, smtpstatus, host, smtpfp, verboselog)
 		  smtp_isopen = 1;
 	      }
 	      smtpstatus = deliver(dp, smtpfp, verboselog, rphead, rp->next);
-	      if (smtpstatus != EX_OK) {
-		fclose(smtpfp[0]); smtpfp[0] = NULL;
-		fclose(smtpfp[1]); smtpfp[1] = NULL;
-		smtp_isopen = 0;
-	      }
+	      /*
+		 if (smtpstatus != EX_OK) {
+		 fclose(smtpfp[0]); smtpfp[0] = NULL;
+		 fclose(smtpfp[1]); smtpfp[1] = NULL;
+		 if (logfp)
+		 fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+		 smtp_isopen = 0;
+		 }
+	       */
 	      rphead = rp->next;
 	    } else {
 	      while (rphead != rp->next) {
@@ -639,7 +649,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    notaryreport(rp->addr->user,notaryacct(r,"*NOTOK*"),NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
 	  }
-	  smtpwrite(smtpfp, verboselog, 0, "RSET");
+	  if (smtp_isopen)
+	    smtpwrite(smtpfp, verboselog, 0, "RSET");
 	  return r;
 	}
 	nrcpt = 0;
@@ -672,6 +683,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	  if (r != EX_OK) {
 	    notaryreport(rp->addr->user, notaryacct(r,"*NOTOK"),NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
+	    if (!smtp_isopen)
+	      break;
 	  } else {
 	    ++nrcpt;
 	    rp->status = EX_OK;
@@ -679,16 +692,21 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	}
 	if (nrcpt == 0) {
 	  /* all the RCPT To addresses were rejected, so reset server */
-	  smtpwrite(smtpfp, verboselog, 0, "RSET");
+	  if (smtp_isopen)
+	    smtpwrite(smtpfp, verboselog, 0, "RSET");
 	  return EX_UNAVAILABLE;
 	}
+	if (!smtp_isopen)
+	  return EX_TEMPFAIL;
+
 	if ((r = smtpwrite(smtpfp, verboselog, 1, "DATA")) != EX_OK) {
 	  for (rp = startrp; rp != endrp; rp = rp->next)
 	    if (rp->status == EX_OK) {
 	      notaryreport(rp->addr->user,notaryacct(r,"*NOTOK"),NULL);
 	      diagnostic(rp, EX_TEMPFAIL, "%s", remotemsg);
 	    }
-	  smtpwrite(smtpfp, verboselog, 0, "RSET");
+	  if (smtp_isopen)
+	    smtpwrite(smtpfp, verboselog, 0, "RSET");
 	  return r;
 	}
 	/* Headers are 7-bit stuff -- says MIME specs */
@@ -716,7 +734,8 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	    }
 	  if (verboselog)
 	    fprintf(verboselog,"Writing headers after DATA failed\n");
-	  smtpwrite(smtpfp, verboselog, 0, "RSET");
+	  if (smtp_isopen)
+	    smtpwrite(smtpfp, verboselog, 0, "RSET");
 	  return EX_TEMPFAIL;
 	}
 	
@@ -742,6 +761,13 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	   * error.  The safe thing to do is to just abort processing.
 	   * Don't send the dot! 2/June/94 edwin@cs.toronto.edu
 	   */
+	  if (smtp_isopen) {
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    if (logfp)
+	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    smtp_isopen = 0;
+	  }
 	  return;
 	}
 	/*
@@ -773,7 +799,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 			 NULL);
 	    diagnostic(rp, r, "%s", remotemsg);
 	  }
-	if (r != EX_OK)
+	if (r != EX_OK && smtp_isopen)
 	  smtpwrite(smtpfp, verboselog, 0, "RSET");
 	return r;
 }
@@ -1266,6 +1292,13 @@ smtpopen(host, fparr, verboselog)
 	  } else {
 	    sprintf(SMTPbuf, "HELO %s", myhostname);
 	    i = smtpwrite(fparr, verboselog, 1, SMTPbuf);
+	    if (i == EX_TEMPFAIL && smtp_isopen) {
+	      fclose(fparr[0]); fparr[0] = NULL;
+	      fclose(fparr[1]); fparr[1] = NULL;
+	      if (logfp)
+		fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	      smtp_isopen = 0;
+	    }
 	  }
 	} while (i == EX_TEMPFAIL && firstmx <= mxcount);
 	if (debug) {
@@ -1286,7 +1319,7 @@ smtpconn(host, fparr, verboselog)
 	FILE *fparr[], *verboselog;
 {
 	int	i, r, retval, smtpfd;
-	u_long	ipaddr;
+	struct in_addr	ipaddr;
 	struct hostent *hp, hent;
 	char	hbuf[MAXHOSTNAMELEN+1], *addrs[2];
 	struct mxdata mxh[MAXFORWARDERS];
@@ -1311,8 +1344,8 @@ smtpconn(host, fparr, verboselog)
 	       ++cp, ++hcp)
 	    *cp = *hcp;
 	  *cp = '\0';
-	  ipaddr = inet_addr(buf);
-	  if (((int)ipaddr) == -1) {
+	  ipaddr.s_addr = inet_addr(buf);
+	  if (ipaddr.s_addr == 0xFFFFFFFF) {
 	    sprintf(remotemsg, "500 (bad IP address: %s)", host);
 	    notaryreport(NULL,NULL,remotemsg);
 	    if (verboselog)
@@ -1325,8 +1358,8 @@ smtpconn(host, fparr, verboselog)
 	  hp->h_name = hbuf;
 	  hp->h_addrtype = AF_INET;
 	  hp->h_aliases = NULL;
-	  hp->h_length = sizeof ipaddr;
-	  addrs[0] = (char *)&ipaddr;
+	  hp->h_length = sizeof(ipaddr);
+	  addrs[0] = (void *)&ipaddr;
 	  addrs[1] = NULL;
 	  hp_setalist(hp, addrs);
 	  retval = makeconn(hp, &smtpfd, verboselog, fparr, mxcnt);
@@ -1737,12 +1770,22 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	      }
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    if (logfp)
+	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else if (r != len) {
 	    sprintf(remotemsg, "500 (SMTP cmd write failure: Only wrote %d of %d bytes!)", r, len);
 	    notaryreport(NULL,NULL,remotemsg);
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    if (logfp)
+	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
 	  if (logfp != NULL) {
@@ -1835,21 +1878,36 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    }
 	  } else if (r == -1) {
 	    if (gotalarm) {
-	      sprintf(remotemsg, "466 (Timeout on %sSMTP responce read, Cmd: %s)",
-		      strbuf == NULL ? "initial ":"", strbuf);
+	      if (strbuf == NULL)
+		sprintf(remotemsg,
+			"466 (Timeout on initial SMTP responce read)");
+	      else
+		sprintf(remotemsg,
+			"466 (Timeout on SMTP responce read, Cmd: %s)",
+			strbuf);
 	      notaryreport(NULL,NULL,remotemsg);
 	    } else
 	      {
 		se = strerror(errno);
-		sprintf(remotemsg, "500 (Error on %sSMTP responce read: %s, Cmd: %s)",
-			strbuf == NULL ? "initial ":"", se,
-			strbuf == NULL ? "(null)" : strbuf);
+		if (strbuf == NULL)
+		  sprintf(remotemsg,
+			  "500 (Error on initial SMTP responce read: %s)",se);
+
+		else
+		  sprintf(remotemsg,
+			  "500 (Error on SMTP responce read: %s, Cmd: %s)",
+			  se, strbuf);
 		notaryreport(NULL,NULL,remotemsg);
 	      }
 	    dflag = 0;
 	    firstmx = 0;
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    if (logfp)
+	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  } else {
 	    sprintf(remotemsg, "500 (Server hung up on us! Cmd: %s)",strbuf);
@@ -1857,6 +1915,11 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	    dflag = 0;
 	    if (verboselog)
 	      fprintf(verboselog,"%s\n",remotemsg);
+	    fclose(smtpfp[0]); smtpfp[0] = NULL;
+	    fclose(smtpfp[1]); smtpfp[1] = NULL;
+	    if (logfp)
+	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	    smtp_isopen = 0;
 	    return EX_TEMPFAIL;
 	  }
 	  /* Exit if the last thing we read was a LF and we're on the
@@ -1870,6 +1933,11 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
+	  fclose(smtpfp[0]); smtpfp[0] = NULL;
+	  fclose(smtpfp[1]); smtpfp[1] = NULL;
+	  if (logfp)
+	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
 	*--cp = '\0';	/* kill the LF */
@@ -1879,6 +1947,11 @@ smtpwrite(smtpfp, verboselog, saverpt, strbuf)
 	  dflag = 0;
 	  if (verboselog)
 	    fprintf(verboselog,"%s\n",remotemsg);
+	  fclose(smtpfp[0]); smtpfp[0] = NULL;
+	  fclose(smtpfp[1]); smtpfp[1] = NULL;
+	  if (logfp)
+	    fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+	  smtp_isopen = 0;
 	  return EX_TEMPFAIL;
 	}
 	--cp;
